package com.ruoyi.ss.device.service;

import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.enums.LoginType;
import com.ruoyi.common.enums.UserType;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.*;
import com.ruoyi.ss.device.domain.SmDevice;
import com.ruoyi.ss.device.domain.SmDeviceDto;
import com.ruoyi.ss.device.domain.SmDeviceVo;
import com.ruoyi.ss.device.domain.enums.DeviceOnlineStatus;
import com.ruoyi.ss.device.domain.enums.DeviceOutageWay;
import com.ruoyi.ss.device.domain.enums.DevicePowerStatus;
import com.ruoyi.ss.device.domain.enums.DeviceStatus;
import com.ruoyi.ss.device.mapper.SmDeviceMapper;
import com.ruoyi.ss.deviceBindRecord.service.ISmDeviceBindRecordService;
import com.ruoyi.ss.deviceGroup.mapper.SmDeviceGroupMapper;
import com.ruoyi.ss.deviceGroup.service.ISmDeviceGroupService;
import com.ruoyi.ss.deviceTenant.domain.SmDeviceTenant;
import com.ruoyi.ss.deviceTenant.mapper.SmDeviceTenantMapper;
import com.ruoyi.ss.deviceTenant.service.ISmDeviceTenantService;
import com.ruoyi.ss.model.mapper.SmModelMapper;
import com.ruoyi.ss.resetRecord.domain.SmResetRecord;
import com.ruoyi.ss.resetRecord.service.ISmResetRecordService;
import com.ruoyi.ss.user.domain.SmUserVo;
import com.ruoyi.ss.user.mapper.SmUserMapper;
import com.ruoyi.iot.domain.IotDeviceInfo;
import com.ruoyi.iot.service.IotService;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 设备Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-02-21
 */
@Service
@Slf4j
public class SmDeviceServiceImpl implements ISmDeviceService
{
    @Autowired
    private SmDeviceMapper smDeviceMapper;

    @Autowired
    private SmDeviceGroupMapper smDeviceGroupMapper;

    @Autowired
    private SmModelMapper smModelMapper;

    @Autowired
    private SmUserMapper smUserMapper;

    @Autowired
    private ISmDeviceTenantService smDeviceTenantService;

    @Autowired
    private SmDeviceTenantMapper smDeviceTenantMapper;

    @Autowired
    private IotService iotService;

    @Autowired
    private ISmDeviceBindRecordService smDeviceBindRecordService;

    @Autowired
    private ISmResetRecordService smResetRecordService;

    @Autowired
    private ISmDeviceGroupService smDeviceGroupService;

    @Autowired
    private ScheduledExecutorService scheduledExecutorService;

    @Autowired
    private DeviceAssembler deviceAssembler;

    @Value("${debug}")
    private Boolean debug;

    /**
     * 查询设备
     *
     * @param deviceId 设备主键
     * @return 设备
     */
    @Override
    public SmDeviceVo selectSmDeviceByDeviceId(Long deviceId)
    {
        SmDeviceVo device = smDeviceMapper.selectSmDeviceByDeviceId(deviceId);
        if (device == null) {
            return null;
        }
        assembleDeviceInfo(Collections.singletonList(device));
        return device;
    }

    /**
     * 查询设备列表
     *
     * @param smDevice 设备
     * @return 设备
     */
    @Override
    public List<SmDeviceVo> selectSmDeviceList(SmDeviceDto smDevice)
    {
        smDevice.setDeleted(false);
        List<SmDeviceVo> list = smDeviceMapper.selectSmDeviceList(smDevice);
        assembleDeviceInfo(list);
        return list;
    }

    /**
     * 拼接设备信息
     * @param list 设备列表
     */
    private void assembleDeviceInfo(List<SmDeviceVo> list) {
        if (CollectionUtils.isEmpty(list)) {
            return;
        }

        // 获取当前设备数据
        List<String> deviceNames = list.stream().map(SmDeviceVo::getMac).filter(StringUtils::hasText).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(deviceNames)) {
            return;
        }
        Map<String, IotDeviceInfo> deviceInfoMap = iotService.getDeviceInfo(deviceNames)
                .stream().collect(Collectors.toMap(IotDeviceInfo::getMac, item -> item));

        // 拼接数据
        for (SmDeviceVo device : list) {
            if (StringUtils.hasText(device.getMac())) {
                // 在线状态
                device.setOnlineStatus(iotService.getOnlineStatus(device.getMac()).getStatus());

                // 其他信息
                IotDeviceInfo deviceInfo = deviceInfoMap.get(device.getMac());
                if (deviceInfo != null) {
                    device.setElectricity(deviceInfo.getA());
                    device.setVoltage(deviceInfo.getV());
                    device.setRealTimePower(deviceInfo.getP());
                    device.setSurplusElectriQuantity(deviceInfo.getM());    // 剩余电量
                    device.setTotalElectriQuantity(deviceInfo.getW());  // 电量
                }
            } else {
                device.setOnlineStatus(DeviceOnlineStatus.OFFLINE.getStatus());
            }
        }
    }

    /**
     * 新增设备
     * F
     * @param data 设备
     * @return 结果
     */
    @Override
    public int insertSmDevice(SmDevice data)
    {
        this.validate(data, false);

        deviceAssembler.setGroup(data); // 房东分组

        data.setCreateTime(DateUtils.getNowDate());
        data.setStatus(DeviceStatus.NORMAL.getStatus());
        return smDeviceMapper.insertSmDevice(data);
    }

    /**
     * 修改设备
     *
     * @param smDevice 设备
     * @return 结果
     */
    @Override
    @Transactional
    public int updateSmDevice(SmDevice smDevice)
    {
        // 校验
        this.validate(smDevice, true);

        // 修改数据库
        smDevice.setUpdateTime(DateUtils.getNowDate());
        int updateCount = smDeviceMapper.updateSmDevice(smDevice);

        // 设置物联网设备信息
        SmDeviceVo device = smDeviceMapper.selectSmDeviceByDeviceId(smDevice.getDeviceId());
        if (!StringUtils.isBlank(device.getMac())) {
            boolean updateDevice = iotService.updateDevice(device);
            ServiceUtil.assertion(!updateDevice, "设备修改失败，请刷新后重试");
        }

        return updateCount;
    }

    private void validate(SmDevice smDevice, boolean update) {
        // 判断是否有权限修改
        LoginUser loginUser = SecurityUtils.getLoginUser();
        if (update) {
            ServiceUtil.assertion(smDevice.getDeviceId() == null, "参数错误：设备id不允许为空");
            SmDevice device = smDeviceMapper.selectSmDeviceByDeviceId(smDevice.getDeviceId());
            ServiceUtil.assertion(device == null || device.getDeleted(), "设备" + smDevice.getDeviceId() + "不存在");
            if (LoginType.FRONT.equals(loginUser.getLoginType())) {
                ServiceUtil.assertion(!Objects.equals(device.getUserId(), loginUser.getUserId()), "这不是您的设备");
            }
        }
    }

    /**
     * 批量删除设备
     *
     * @param deviceIds 需要删除的设备主键
     * @return 结果
     */
    @Override
    public int deleteSmDeviceByDeviceIds(Long[] deviceIds)
    {
        this.validatePreDelete(Arrays.asList(deviceIds));
        return smDeviceMapper.deleteSmDeviceByDeviceIds(deviceIds);
    }

    /**
     * 真正的删除校验
     */
    private void validatePreDelete(List<Long> deviceIds) {
        ServiceUtil.assertion(CollectionUtils.isEmpty(deviceIds), "设备不能为空");
        SmDeviceDto dto = new SmDeviceDto();
        dto.setDeviceIds(deviceIds);
        dto.setDeleted(true);
        Map<Long, SmDeviceVo> deviceMap = selectMap(dto);

        for (Long deviceId : deviceIds) {
            SmDeviceVo device = deviceMap.get(deviceId);
            ServiceUtil.assertion(device == null, String.format("设备“%s”不存在，请刷新后重试", deviceId));
        }
    }

    /**
     * 逻辑删除校验
     */
    private void validatePreLogicDelete(List<Long> deviceIds) {
        ServiceUtil.assertion(CollectionUtils.isEmpty(deviceIds), "设备不能为空");

        SmDeviceDto dto = new SmDeviceDto();
        dto.setDeviceIds(deviceIds);
        dto.setDeleted(false);
        // 前台用户，判断是不是他的设备
        if (LoginType.FRONT.equals(SecurityUtils.getLoginType())) {
            dto.setUserId(SecurityUtils.getUserId());
        }
        Map<Long, SmDeviceVo> deviceMap = selectMap(dto);

        deviceAssembler.assembleTenant(new ArrayList<>(deviceMap.values()));
        for (Long deviceId : deviceIds) {
            SmDeviceVo device = deviceMap.get(deviceId);
            ServiceUtil.assertion(device == null, String.format("设备“%s”不存在，请刷新后重试", deviceId));
            ServiceUtil.assertion(!CollectionUtils.isEmpty(device.getTenantIds()), String.format("设备“%s”还有租户，请解绑后再进行删除", device.getDeviceName()));
        }

    }

    /**
     * 条件查询列表，并转为Map
     */
    private Map<Long, SmDeviceVo> selectMap(SmDeviceDto dto) {
        List<SmDeviceVo> list = smDeviceMapper.selectSmDeviceList(dto);
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyMap();
        }
        return list.stream().collect(Collectors.toMap(SmDeviceVo::getDeviceId, item -> item));
    }

    @Override
    public int selectCount(SmDeviceDto dto) {
        return smDeviceMapper.selectCount(dto);
    }

    @Override
    public List<SmDeviceVo> selectSmDeviceByGroupId(Long groupId) {
        SmDeviceDto dto = new SmDeviceDto();
        dto.setGroupId(groupId);
        dto.setDeleted(false);
        return smDeviceMapper.selectSmDeviceList(dto);
    }

    @Override
    @Transactional
    public boolean addElectricity(Long deviceId, BigDecimal num) {

        ServiceUtil.assertion(num.compareTo(BigDecimal.ZERO) < 0, "增加的电量不允许小于0");

        SmDeviceVo device = smDeviceMapper.selectSmDeviceByDeviceId(deviceId);
        ServiceUtil.assertion(device == null, "设备不存在");
        ServiceUtil.assertion(!StringUtils.hasText(device.getMac()), "设备MAC号为空");

        // 更新数据库电量
        int updateCount = smDeviceMapper.addElectricity(deviceId, num);
        ServiceUtil.assertion(updateCount != 1, "增加电量失败，请刷新后重试");

        // 物联网设备增加电量
        boolean rechargeResult = iotService.recharge(device.getMac(), num);
        ServiceUtil.assertion(!rechargeResult, "设备充值失败");

        // 拉取设备信息
        this.pullDeviceInfoAsync(Collections.singletonList(deviceId), 3, TimeUnit.SECONDS);

        return true;
    }

    @Override
    @Transactional
    public boolean tenantBind(Long tenantId, Long deviceId) {
        // 校验
        ServiceUtil.assertion(tenantId == null || deviceId == null, "参数错误");
        ServiceUtil.assertion(smDeviceTenantService.selectIsBind(tenantId, deviceId), "用户与该设备已经绑定，无需重复绑定");

        SmDeviceVo device = smDeviceMapper.selectSmDeviceByDeviceId(deviceId);
        ServiceUtil.assertion(device == null || device.getDeleted(), "待绑定的设备不存在");
        ServiceUtil.assertion(device.getUserId() == null, "该设备还未绑定房东，请先绑定房东");

        SmUserVo user = smUserMapper.selectSmUserByUserId(tenantId);
        ServiceUtil.assertion(user == null, "用户不存在");
        ServiceUtil.assertion(!UserType.TENANT.equalsType(user.getUserType()), "用户身份不是租户，无法绑定");

        // 执行绑定
        SmDeviceTenant deviceTenant = new SmDeviceTenant();
        deviceTenant.setTenantId(tenantId);
        deviceTenant.setDeviceId(deviceId);
        int insertCount = smDeviceTenantService.insertSmDeviceTenant(deviceTenant);
        ServiceUtil.assertion(insertCount != 1 , "绑定失败，请刷新后重试");

        // 设置为默认设备
        setDefaultDevice(tenantId, deviceId);

        return true;
    }

    @Override
    @Transactional
    public boolean tenantUnbind(Long tenantId, Long deviceId) {
        // 判断是否已绑定
        ServiceUtil.assertion(!smDeviceTenantService.selectIsBind(tenantId, deviceId), "用户尚未绑定该设备，无法解绑");

        // 执行解绑
        int deleteCount = smDeviceTenantService.unBind(tenantId, deviceId);
        ServiceUtil.assertion(deleteCount != 1, "设备解绑失败，请刷新后重试");

        // 若解绑的设备为默认设备，则设置新的默认设备
        SmDeviceVo defaultDevice = selectDefaultDevice(tenantId);
        if (defaultDevice == null) {
            List<SmDeviceVo> deviceList = this.selectTenantDeviceList(tenantId);
            if (!CollectionUtils.isEmpty(deviceList)) {
                setDefaultDevice(tenantId, deviceList.get(0).getDeviceId());
            }
        }

        return true;
    }

    /**
     * 获取租户设备列表
     * @param tenantId 租户id
     * @return
     */
    private List<SmDeviceVo> selectTenantDeviceList(Long tenantId) {
        if (tenantId == null) {
            return Collections.emptyList();
        }
        SmDeviceDto dto = new SmDeviceDto();
        dto.setTenantId(tenantId);
        dto.setDeleted(false);
        List<SmDeviceVo> list = smDeviceMapper.selectSmDeviceList(dto);
        assembleDeviceInfo(list);
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyList();
        }
        return list;
    }

    @Override
    public SmDeviceVo selectDefaultDevice(Long tenantId) {
        if (tenantId == null) {
            return null;
        }
        SmDeviceDto dto = new SmDeviceDto();
        dto.setTenantId(tenantId);
        dto.setIsDefault(true);
        dto.setDeleted(false);
        List<SmDeviceVo> list = smDeviceMapper.selectSmDeviceList(dto);
        assembleDeviceInfo(list);
        if (CollectionUtils.isEmpty(list)) {
            return null;
        }
        return list.get(0);
    }

    @Override
    @Transactional
    public boolean setDefaultDevice(Long tenantId, Long deviceId) {
        SmDeviceVo device = smDeviceMapper.selectSmDeviceByDeviceId(deviceId);
        ServiceUtil.assertion(device == null || device.getDeleted(), "设备不存在");

        deviceAssembler.assembleTenant(Collections.singletonList(device));
        ServiceUtil.assertion(!device.getTenantIds().contains(tenantId), "该用户不是这个设备的租户");
        SmUserVo user = smUserMapper.selectSmUserByUserId(tenantId);
        ServiceUtil.assertion(user == null, "用户不存在");

        int updateCount = smDeviceTenantMapper.setDefault(tenantId, deviceId);
        ServiceUtil.assertion(updateCount == 0, "设置默认设备失败");
        return true;
    }

    @Override
    public boolean logicDel(List<Long> deviceIds) {
        this.validatePreLogicDelete(deviceIds);
        return smDeviceMapper.logicDel(deviceIds) > 0;
    }

    @Override
    public Map<Long, List<SmDeviceVo>> selectGroupByModelIds(List<Long> modelIds) {
        if (CollectionUtils.isEmpty(modelIds)) {
            return Collections.emptyMap();
        }
        SmDeviceDto dto = new SmDeviceDto();
        dto.setDeleted(false);
        dto.setModelIds(modelIds);
        List<SmDeviceVo> list = smDeviceMapper.selectSmDeviceList(dto);
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyMap();
        }
        return list.stream().collect(Collectors.groupingBy(SmDeviceVo::getModelId));
    }

    /**
     * 切换设备通电状态（通/断电）
     * @param deviceId 设备id
     * @param status 通电状态
     */
    @Override
    public boolean changePowerStatus(Long deviceId, DevicePowerStatus status) {
        SmDeviceVo device = smDeviceMapper.selectSmDeviceByDeviceId(deviceId);
        ServiceUtil.assertion(device == null || device.getDeleted(), "设备不存在");
        ServiceUtil.assertion(!StringUtils.hasText(device.getMac()), "设备Mac号为空");

        deviceAssembler.assembleTenant(Collections.singletonList(device));
        ServiceUtil.assertion(!UserUtil.hasFrontUser(device.getTenantIds())
                && !UserUtil.hasFrontUser(device.getUserId()), "您不是该设备的租户或房东，无法进行该操作" );

        if (DevicePowerStatus.ON.equals(status)) {
            // 通电，检查电量余额是否足够
            IotDeviceInfo deviceInfo = iotService.getDeviceInfo(device.getMac());
            ServiceUtil.assertion(deviceInfo.getM() == null
                    || BigDecimal.ZERO.compareTo(deviceInfo.getM()) >= 0, "电量余额不足，请充值后重试");

            iotService.open(device.getMac());
        } else if (DevicePowerStatus.OFF.equals(status)) {
            iotService.close(device.getMac());
        } else {
            throw new ServiceException("不支持的操作");
        }

        // 修改数据库通断电状态
        SmDevice changePowerForm = new SmDevice();
        changePowerForm.setDeviceId(deviceId);
        changePowerForm.setPowerStatus(status.getStatus());
        smDeviceMapper.updateSmDevice(changePowerForm);

        return true;
    }

    /**
     * 拉取设备信息，并保存到数据库
     * @param deviceIds 设备id列表（为空则查询所有）
     */
    @Override
    @SneakyThrows
    public void pullDeviceInfo(List<Long> deviceIds) {
        // 查询设备列表
        SmDeviceDto dto = new SmDeviceDto();
        dto.setDeviceIds(deviceIds);
        List<SmDeviceVo> list = smDeviceMapper.selectSimpleList(dto);
        if (CollectionUtils.isEmpty(list)) {
            return;
        }

        // 获取数据点信息
        List<String> deviceNames = list.stream().map(SmDeviceVo::getMac).filter(StringUtils::hasText).collect(Collectors.toList());
        List<IotDeviceInfo> deviceInfos = iotService.getDeviceInfo(deviceNames);

        // 更新数据
        for (SmDeviceVo device : list) {
            try {
                IotDeviceInfo deviceInfo = deviceInfos.stream().filter(item -> Objects.equals(device.getMac(), item.getMac())).findFirst().orElse(null);
                SmDevice data = new SmDevice();
                data.setDeviceId(device.getDeviceId());
                if (deviceInfo != null) {
                    data.setSurplusElectriQuantity(deviceInfo.getM());
                    data.setLastPullTime(deviceInfo.getAt());
                    data.setTotalElectriQuantity(deviceInfo.getW());
                    data.setPowerStatus(deviceInfo.getS());
                    // 更新电表状态
                    if (BigDecimal.ZERO.compareTo(deviceInfo.getM()) >= 0) {
                        if (DeviceOutageWay.IMMEDIATE.getValue().equals(device.getOutageWay())) {
                            // 欠费断电
                            data.setStatus(DeviceStatus.ARREARS_OFF.getStatus());
                        } else {
                            // 欠费
                            data.setStatus(DeviceStatus.ARREARS.getStatus());
                        }
                    } else {
                        // 正常
                        data.setStatus(DeviceStatus.NORMAL.getStatus());
                    }
                }

                // 是否在线
                data.setOnlineStatus(iotService.getOnlineStatus(device.getMac()).getStatus());

                smDeviceMapper.updateSmDevice(data);
            } catch (Exception e) {
                log.error("更新设备信息失败，deviceId:{}", device.getDeviceId(), e);
            }
        }
    }

    /**
     * 异步延迟执行拉取设备信息
     * @param deviceIds 设备id列表
     * @param delay 延迟时间
     * @param unit 时间单位
     */
    private void pullDeviceInfoAsync(List<Long> deviceIds, long delay, TimeUnit unit) {
        scheduledExecutorService.schedule(() -> {
            log.info("异步拉取设备信息 start");
            this.pullDeviceInfo(deviceIds);
            log.info("异步拉取设备信息 end");
        }, delay, unit);
    }

    @Override
    @Transactional
    public boolean bind(Long userId, Long deviceId) {
        ServiceUtil.assertion(userId == null || deviceId == null, "绑定设备参数错误");

        SmDeviceVo device = smDeviceMapper.selectSmDeviceByDeviceId(deviceId);
        ServiceUtil.assertion(device == null || device.getDeleted(), "设备不存在");

        if (device.getUserId() == null) {
            // 房东为空，绑定房东
            this.bindLandlord(userId, deviceId);
        } else {
            // 绑定租户
            this.tenantBind(userId, deviceId);
        }
        return true;
    }

    @Override
    @Transactional
    public boolean bind(Long userId, String mac) {
        // 判断设备是否存在
        SmDeviceVo device = selectByMac(mac);

        // 不存在则新增
        if (device == null || device.getDeleted()) {
            device = new SmDeviceVo();
            device.setMac(mac);
            this.insertSmDevice(device);
        }

        // 绑定
        return bind(userId, device.getDeviceId());
    }

    /**
     * 根据mac号查询数据
     * @param mac
     * @return
     */
    @Override
    public SmDeviceVo selectByMac(String mac) {
        if (StringUtils.isBlank(mac)) {
            return null;
        }
        SmDeviceDto dto = new SmDeviceDto();
        dto.setMac(mac);
        List<SmDeviceVo> list = smDeviceMapper.selectSmDeviceList(dto);
        assembleDeviceInfo(list);
        if (CollectionUtils.isEmpty(list)) {
            return null;
        }
        return list.get(0);
    }

    @Override
    @Transactional
    public boolean reset(Long deviceId) {
        ServiceUtil.assertion(deviceId == null, "设备id不能为空");

        SmDeviceVo device = smDeviceMapper.selectSmDeviceByDeviceId(deviceId);
        ServiceUtil.assertion(device == null || device.getDeleted(), "设备不存在");
        ServiceUtil.assertion(!UserUtil.hasFrontUser(device.getUserId()), "您不是该设备的房东，无法进行该操作");
        ServiceUtil.assertion(hasTenant(deviceId), "该设备还有租户，不允许归零");

        // 电量归零（设置初始读数）
        SmDevice form = new SmDevice();
        form.setDeviceId(deviceId);
        form.setInitReading(device.getTotalElectriQuantity());
        smDeviceMapper.updateSmDevice(form);

        // 归零记录
        scheduledExecutorService.schedule(() -> {
            SmResetRecord reset = new SmResetRecord();
            reset.setDeviceId(deviceId);
            reset.setLandlordId(device.getUserId());
            reset.setSurplusElectriQuantity(device.getSurplusElectriQuantity());
            reset.setTotalElectriQuantity(device.getTotalElectriQuantity());
            smResetRecordService.insertSmResetRecord(reset);
        }, 0, TimeUnit.SECONDS);

        return true;
    }

    /**
     * 判断设备是否还有租户
     * @param deviceId
     * @return
     */
    private boolean hasTenant(Long deviceId) {
        if (deviceId == null) {
            return false;
        }
        List<SmDeviceTenant> tenantList = smDeviceTenantService.selectByDeviceId(deviceId);
        return !CollectionUtils.isEmpty(tenantList);
    }

    /**
     * 设备是否已经被绑定
     *
     * @param mac
     * @return
     */
    @Override
    public boolean isBind(String mac) {
        if (StringUtils.isBlank(mac)) {
            return false;
        }
        SmDeviceVo device = smDeviceMapper.selectSimpleSmDeviceByMac(mac);
        return device != null && device.getUserId() != null;
    }

    @Override
    @Transactional
    public boolean landlordUnbind(Long userId, Long deviceId) {
        SmDeviceVo device = smDeviceMapper.selectSmDeviceByDeviceId(deviceId);
        ServiceUtil.assertion(device == null || device.getDeleted(), "设备不存在");
        ServiceUtil.assertion(!Objects.equals(device.getUserId(), userId), "您不是该设备的房东");

        deviceAssembler.assembleTenant(Collections.singletonList(device));
        ServiceUtil.assertion(!CollectionUtils.isEmpty(device.getTenantIds()), "该设备还有租户绑定，无法解除绑定");

        int updateCount = smDeviceMapper.unbindLandlord(deviceId, userId);
        ServiceUtil.assertion(updateCount != 1, "当前设备信息已变更，请刷新后重试");
        return true;
    }

    // 绑定房东
    @Transactional
    public void bindLandlord(Long userId, Long deviceId) {
        ServiceUtil.assertion(userId == null || deviceId == null, "房东绑定设备参数错误");

        SmUserVo user = smUserMapper.selectSmUserByUserId(userId);
        ServiceUtil.assertion(!UserType.LANDLORD.equalsType(user.getUserType()), "用户身份不是房东");

        int updateCount = smDeviceMapper.bindLandlord(deviceId, userId);
        ServiceUtil.assertion(updateCount != 1, "当前设备信息已变更，请刷新后重试");

        // 更新设备分组
        SmDevice device = new SmDevice();
        device.setDeviceId(deviceId);
        device.setUserId(userId);
        deviceAssembler.setGroup(device);
        smDeviceMapper.updateSmDevice(device);

        // 记录设备绑定
        scheduledExecutorService.schedule(()-> {
            smDeviceBindRecordService.record(userId, deviceId);
        },0, TimeUnit.SECONDS);
    }
}
