package com.ruoyi.ss.deviceGroup.service;

import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.enums.LoginType;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.ss.device.domain.SmDeviceDto;
import com.ruoyi.ss.device.domain.SmDeviceVo;
import com.ruoyi.ss.device.service.ISmDeviceService;
import com.ruoyi.ss.deviceGroup.domain.SmDeviceGroup;
import com.ruoyi.ss.deviceGroup.domain.SmDeviceGroupVo;
import com.ruoyi.ss.deviceGroup.mapper.SmDeviceGroupMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 设备分组Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-02-21
 */
@Service
public class SmDeviceGroupServiceImpl implements ISmDeviceGroupService
{
    @Autowired
    private SmDeviceGroupMapper smDeviceGroupMapper;

    @Autowired
    private ISmDeviceService smDeviceService;

    /**
     * 查询设备分组
     *
     * @param groupId 设备分组主键
     * @return 设备分组
     */
    @Override
    public SmDeviceGroup selectSmDeviceGroupByGroupId(Long groupId)
    {
        return smDeviceGroupMapper.selectSmDeviceGroupByGroupId(groupId);
    }

    /**
     * 查询设备分组列表
     *
     * @param smDeviceGroup 设备分组
     * @return 设备分组
     */
    @Override
    public List<SmDeviceGroupVo> selectSmDeviceGroupList(SmDeviceGroup smDeviceGroup)
    {
        return smDeviceGroupMapper.selectSmDeviceGroupList(smDeviceGroup);
    }

    /**
     * 新增设备分组
     *
     * @param smDeviceGroup 设备分组
     * @return 结果
     */
    @Override
    public int insertSmDeviceGroup(SmDeviceGroup smDeviceGroup)
    {
        smDeviceGroup.setCreateTime(DateUtils.getNowDate());
        return smDeviceGroupMapper.insertSmDeviceGroup(smDeviceGroup);
    }

    /**
     * 修改设备分组
     *
     * @param smDeviceGroup 设备分组
     * @return 结果
     */
    @Override
    public int updateSmDeviceGroup(SmDeviceGroup smDeviceGroup)
    {
        this.validate(Collections.singletonList(smDeviceGroup), true);
        return smDeviceGroupMapper.updateSmDeviceGroup(smDeviceGroup);
    }

    /**
     * 批量删除设备分组
     *
     * @param groupIds 需要删除的设备分组主键
     * @return 结果
     */
    @Override
    public int deleteSmDeviceGroupByGroupIds(Long[] groupIds)
    {
        this.validatePreDelete(Arrays.asList(groupIds));
        return smDeviceGroupMapper.deleteSmDeviceGroupByGroupIds(groupIds);
    }

    /**
     * 删除设备分组信息
     *
     * @param groupId 设备分组主键
     * @return 结果
     */
    @Override
    @Transactional
    public int deleteSmDeviceGroupByGroupId(Long groupId) {
        // 删除校验
        this.validatePreDelete(groupId);

        // 删除分组
        return smDeviceGroupMapper.deleteSmDeviceGroupByGroupId(groupId);
    }

    /**
     * 校验数据
     */
    private void validate(List<SmDeviceGroup> list, boolean update) {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        if (LoginType.FRONT.equals(loginUser.getLoginType())) {
            if (update) {
                List<SmDeviceGroupVo> groups = smDeviceGroupMapper.selectSmDeviceGroupByGroupIds(list.stream().map(SmDeviceGroup::getGroupId).distinct().collect(Collectors.toList()));
                Long userId = SecurityUtils.getUserId();
                groups.forEach(item -> {
                    ServiceUtil.assertion(!Objects.equals(item.getUserId(), userId), String.format("存在不属于您的分组：%s", item.getGroupName()));
                });
            }
        }
    }

    @Override
    @Transactional
    public boolean changeSort(List<SmDeviceGroup> list) {
        ServiceUtil.assertion(CollectionUtils.isEmpty(list), "修改排序的列表不允许为空");
        this.validate(list, true);

        List<SmDeviceGroup> updateList = new ArrayList<>();
        for (SmDeviceGroup item : list) {
            SmDeviceGroup group = new SmDeviceGroup();
            group.setGroupId(item.getGroupId());
            group.setGroupSort(item.getGroupSort());
            updateList.add(group);
        }
        smDeviceGroupMapper.batchUpdate(updateList);
        return true;
    }

    @Override
    public List<SmDeviceGroupVo> listCount(SmDeviceGroup smDeviceGroup) {
        List<SmDeviceGroupVo> list = new ArrayList<>();

        // 查询全部设备数据
        SmDeviceDto deviceDto = new SmDeviceDto();
        deviceDto.setUserId(smDeviceGroup.getUserId());
        int count = smDeviceService.selectCount(deviceDto);
        SmDeviceGroupVo all = new SmDeviceGroupVo();
        all.setDeviceCount(count);
        all.setGroupName("全部");
        list.add(all);

        // 查询各各分组的设备数据
        list.addAll(smDeviceGroupMapper.selectSmDeviceGroupList(smDeviceGroup));
        return list;
    }

    @Override
    public SmDeviceGroupVo selectDefaultGroup(Long userId) {
        if (userId == null) {
            return null;
        }
        SmDeviceGroup dto = new SmDeviceGroup();
        dto.setIsDefault(true);
        dto.setUserId(userId);
        List<SmDeviceGroupVo> list = selectSmDeviceGroupList(dto);
        if (CollectionUtils.isEmpty(list)) {
            return null;
        }
        return list.get(0);
    }

    @Override
    @Transactional
    public SmDeviceGroupVo insertDefaultGroup(Long userId) {
        SmDeviceGroupVo group = new SmDeviceGroupVo();
        group.setUserId(userId);
        group.setGroupName("未分组");
        group.setGroupSort(1);
        group.setIsDefault(true);
        if (this.insertSmDeviceGroup(group) == 1)  {
            return group;
        }
        return null;
    }

    private void validatePreDelete(List<Long> groupIds) {
        if (!CollectionUtils.isEmpty(groupIds)) {
            for (Long groupId : groupIds) {
                this.validatePreDelete(groupId);
            }
        }
    }

    // 删除校验
    private void validatePreDelete(Long groupId) {
        SmDeviceGroup dto = new SmDeviceGroup();
        if (LoginType.FRONT.equals(SecurityUtils.getLoginType())) {
            // 前台用户，判断是不是他的分组
            dto.setUserId(SecurityUtils.getUserId());
        }
        SmDeviceGroupVo group = smDeviceGroupMapper.selectSmDeviceGroupByGroupId(groupId);
        ServiceUtil.assertion(group == null, "分组不存在，请刷新后重试");
        ServiceUtil.assertion(group.getIsDefault(), "默认分组不允许删除");

        // 查询分组下是否含有设备
        List<SmDeviceVo> deviceList = smDeviceService.selectSmDeviceByGroupId(groupId);
        ServiceUtil.assertion(!CollectionUtils.isEmpty(deviceList), "该分组下存在设备，不允许删除");
    }


}
